package conferencia;

import java.util.*;

public class Conferencia {
	
	private static Conferencia conferencia;
	private static int identArtigo = 1;
	
    private String nome;
    private String local;
    private Date data;
    private int numMaxArtigos;
    private Date dataInicioSubmissao;
    private Date dataTerminoSubmissao;
    private Date dataTerminoRevisao;

    private List<Topico> topicos;
    private ChefeComite chefe;
    private List<Avaliador> avaliadores;
    private List<Autor> autores;
    private List<Artigo> artigos;
	
	public static Conferencia getInstance (){
		if (conferencia == null)
			conferencia = new Conferencia ();
		return conferencia;	 
	}
	
    private Conferencia (){
		this.topicos = new ArrayList<Topico>();
		this.avaliadores = new ArrayList<Avaliador>();
	    this.autores = new ArrayList<Autor>();
		this.artigos = new ArrayList<Artigo>();
    }

	public String getNome() {
		return nome;
	}

	public Date getData() {
		return data;
	}

	public Date getDataInicioSubmissao() {
		return dataInicioSubmissao;
	}

	public Date getDataTerminoRevisao() {
		return dataTerminoRevisao;
	}

	public Date getDataTerminoSubmissao() {
		return dataTerminoSubmissao;
	}

	public String getLocal() {
		return local;
	}

	public int getNumMaxArtigos() {
		return numMaxArtigos;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public void setDataInicioSubmissao(Date dataInicioSubmissao) {
		this.dataInicioSubmissao = dataInicioSubmissao;
	}

	public void setDataTerminoRevisao(Date dataTerminoRevisao) {
		this.dataTerminoRevisao = dataTerminoRevisao;
	}

	public void setDataTerminoSubmissao(Date dataTerminoSubmissao) {
		this.dataTerminoSubmissao = dataTerminoSubmissao;
	}

	public void setLocal(String local) {
		this.local = local;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNumMaxArtigos(int numMaxArtigos) {
		this.numMaxArtigos = numMaxArtigos;
	}

    public void iniciaConferencia (String nome, String local, Date data, 
            int numMaxArtigos, Date dataInicioSubmissao, Date dataTerminoSubmissao, 
            Date dataTerminoRevisao, String[] nomesTopicos){
    	this.nome = nome;
    	this.local = local;
    	this.data = data;
    	this.numMaxArtigos = numMaxArtigos;
    	this.dataInicioSubmissao = dataInicioSubmissao;
    	this.dataTerminoSubmissao = dataTerminoSubmissao;
    	this.dataTerminoRevisao = dataTerminoRevisao;
    	for (int i=0; i<nomesTopicos.length; i++){		
    		Topico topico = new Topico ();
    		topico.setNome((String)nomesTopicos[i]);
   			this.topicos.add(topico);
    	}
    }

    public void defineChefe(String nome, String login, String senha){
    	ChefeComite.inicializaClasseChefeComite(nome);
    	this.chefe = ChefeComite.getInstance();
    	this.chefe.setLogin(login);
    	this.chefe.setSenha(senha);
    }

    public void insereAvaliador(String nome, String login, String senha, 
                      String email, String origem, String[] nomesTopicos){

    		...COMPLETAR...
    	
    }

    public void mandaEmail(){}

    public void submeteArtigo(String nome, String resumo, String nomeArquivo,
 				String[] nomesAutores, String[] instituicoesAutores, String[] nomesTopicos){
    	Artigo artigo = new Artigo(nome);
    	artigo.setResumo(resumo);
    	artigo.setIdentificador(identArtigo);
    	artigo.setNomeArquivo(nomeArquivo);
    	identArtigo++;
    	for (int i=0; i<nomesAutores.length; i++){
    		String nomeAutor = (String)nomesAutores[i];
    		String instituicaoAutor = (String)instituicoesAutores[i];
    		Autor autor = new Autor (nomeAutor,instituicaoAutor);
    		int indice = this.autores.indexOf(autor);
    		if (indice == -1)
    			this.autores.add(autor);
    		else
    			autor = this.autores.get(indice);
    		artigo.associaAutor(autor);		
    	}
    		
    	for (int i=0; i<nomesTopicos.length; i++){
    		String nomeTopico = (String)nomesTopicos[i];	
    		Topico topico = new Topico();
    		topico.setNome(nomeTopico);
    		int indice = this.topicos.indexOf(topico);
    		if (indice != -1){
    			topico = this.topicos.get(indice);
	    		artigo.associaTopico(topico);
    		}		
    	}
    	this.artigos.add(artigo);					
    }

    public List<Artigo> selecionaArtigosSemRevisao(){
    // Retorna uma lista contendo os artigos que ainda nao possuem nenhuma revisao		
    	List<Artigo> artigos = new ArrayList<Artigo>();
    	for (int i=0; i<this.artigos.size(); i++){
    		Artigo artigo = this.artigos.get(i);
    		if (!artigo.isTemRevisao())
    			artigos.add(artigo);
    	}
		return artigos;
    }

    public List<Avaliador> selecionaAvaliadoresSemConflito(String nomeArtigo){
    // Retorna uma lista contendo os avaliadores sem conflito com um artigo.		
    	Artigo artigo = new Artigo(nomeArtigo);
    	int indiceArtigo = this.artigos.indexOf(artigo);
    	if (indiceArtigo!= -1){
			artigo = this.artigos.get(indiceArtigo);    		
    		List<Avaliador> avaliadoresConflitantes = artigo.getRevisoresComConflito();
	    	List<Avaliador> avaliadores =  new ArrayList<Avaliador>();
	    	for (int i=0; i<this.avaliadores.size(); i++){
	    		Avaliador avaliador = this.avaliadores.get(i);
	    		if (!avaliadoresConflitantes.contains(avaliador))
	    			avaliadores.add(avaliador);
	    	}
			return avaliadores;
    	}
		else
        	throw new RuntimeException ("artigo inexistente");
    }

    public void defineConflito(String nomeArtigo, String nomeAvaliador){
    	Artigo artigo = new Artigo(nomeArtigo);
    	int indiceArtigo = this.artigos.indexOf(artigo);
    	if (indiceArtigo!= -1){
    		Avaliador avaliador = new Avaliador(nomeAvaliador,"","","","");
    		int indiceAvaliador = this.avaliadores.indexOf(avaliador);
    		if (indiceAvaliador!=-1){
    			artigo = this.artigos.get(indiceArtigo);
    			avaliador = this.avaliadores.get(indiceAvaliador);
    			artigo.defineConflito(avaliador);
    		}
    	}
    }

    public void distribuiArtigos(){

		...COMPLETAR...
	
    }

    public Artigo selecionaArtigo(String nome){
    	Artigo artigo = new Artigo(nome);
    	int indice = this.artigos.indexOf(artigo);
    	if (indice!=-1)
    		return this.artigos.get(indice);
    	else
    		return null;
    }

    public void forneceRevisao(String nomeArtigo, String nomeAvaliador, 
       		 double clareza, double relevancia, double corretude, 
             double notaFinal, String comentariosComite, String comentariosAutor) {
   		Avaliador avaliador = new Avaliador(nomeAvaliador,"","","","");
   		int indiceAvaliador = this.avaliadores.indexOf(avaliador);
   		if (indiceAvaliador!=-1){
   			avaliador = this.avaliadores.get(indiceAvaliador);
   			List<Artigo> artigos = avaliador.retornaArtigosAvaliados();
   			Artigo artigo = new Artigo(nomeArtigo);
   			int indice = artigos.indexOf(artigo);
   			if (indice!=-1){
   				artigo = artigos.get(indice);
   				List<Revisao> revisoes = artigo.getRevisoes();
   				for (int i=0; i<revisoes.size(); i++){
   					Revisao revisao = revisoes.get(i);
   					if (revisao.getAvaliador().equals(avaliador)){
   						revisao.setClareza(clareza);
   						revisao.setRelevancia(relevancia);
   						revisao.setCorretude(corretude);
   						revisao.setNotaFinal(notaFinal);
   						revisao.setComentariosComite(comentariosComite);
   						revisao.setComentariosAutor(comentariosAutor);
   					}	
   				}
   			}
   		}             	
    }
    
    public List<Artigo> selecionaArtigos(){
    // Seleciona todos os artigos
    	return this.artigos;
    }

    public List<Avaliador> selecionaAvaliadores(){
    // Seleciona todos os avaliadores
    	return this.avaliadores;
    }
    
    public List<Artigo> selecionaArtigosComNotas(){
    	for (int i=0; i<this.artigos.size(); i++)
    		(this.artigos.get(i)).calculaNotaFinal();
    	return this.artigos;	
    }

    public void defineArtigosAceitos(String[] nomesArtigos){
    	for (int i=0; i<nomesArtigos.length;i++){
	    	Artigo artigo = new Artigo ((String)nomesArtigos[i]);
	    	int indice = this.artigos.indexOf(artigo);
	    	if (indice!=-1){
	    		artigo = this.artigos.get(indice);
	    		artigo.defineComoAceito();
	    	}
    	}	
    }

    public void modificaStatusArtigo(String nomeArtigo){
    	Artigo artigo = new Artigo(nomeArtigo);
    	int indice = this.artigos.indexOf(artigo);
    	if (indice!=-1){
    		artigo = this.artigos.get(indice);
    		artigo.modificaStatus();
    	}
    }
}


